//+------------------------------------------------------------------+
//|                                Copyright 2020, Waldir A. Pedroso |
//+------------------------------------------------------------------+
#define  robot_name      "Distancia da Media"

#property copyright     "Copyright 2020, Waldir A. Pedroso"
#property version       "1.0"
#property description   "Distancia da Media"


#include <Trade\Trade.mqh>
#include <Trade\PositionInfo.mqh>
#include <Controls\Dialog.mqh>

#include <Math\Alglib\alglib.mqh>
#include <Arrays\ArrayDouble.mqh>
#include <Trade\PositionInfo.mqh>
#include <Trade\DealInfo.mqh>

#include <..\experts\Distancia da Media\Enum.mqh>
#include <..\experts\Distancia da Media\Logger.mqh>


//+------------------------------------------------------------------+
//|                     INPUTS                                       |
//+------------------------------------------------------------------+
input string               Secao0 = "###############";//### Config de Estratégia ###
input ENUM_TIMEFRAMES      Periodo = PERIOD_CURRENT;
input int                  NumeroMagico=0; //O número mágico é utilizado para diferenciar ordens de outros robôs
input ENUM_LOGIC           IsDebug=0;//Mostrar mensagens de debug?
input ENUM_LOGIC           IsStats=1;//Mostrar estatisticas no grafico?
input double               Volume=1; //Volume
input ENUM_LAST_PRICE_TYPE TipoUltimoPreco=0;//Tipo de referência do ultimo preço
input int                  StopGainEmTicks=0;//TP em Pontos (Saida da 1ª Parcial se ativo)
input int                  StopLossEmTicks=0;//SL em Pontos
input int                  TempoTrava=4; //Aguardar x Candles para abrir a proxima ordem
input ENUM_LOGIC           FechaPosMA=1;//Fechar posição ao encostar na media?
input double               QTDEXEC=0; //Qtd de execuções no dia

input string               Secao1 = "###############";//### Range de Distancia da Media ###
input double               DistMAMin=0; //Distancia minima do preço para a media (pts)
input double               DistMAMax=0; //Distancia Maxima do preço para a media (pts)

input string               Secao2 = "###############";//### Horario da Estrategia ###
input ENUM_HORARIO         HoraInicio=0;//Hora de início da estratégia
input ENUM_HORARIO         HoraFim=99;//Hora de término da estratégia
input ENUM_HORARIO         HoraInicioIntervalo=108;//Hora de início do Intervalo da estratégia
input ENUM_HORARIO         HoraFimIntervalo=108;//Hora de término do Intervalo da estratégia
input ENUM_LOGIC           FecharPosition=1;//Fechar posições ao término de horario de execução?

input string               Secao3 = "###############";//### INDICADORES > Media Movel 1 ###
input int                  MA1Periodo=5;//Periodo
input int                  MA1Deslocar=5;//Deslocar
input ENUM_MA_METHOD       MA1Metodo=MODE_EMA;//Metodo
input ENUM_APPLIED_PRICE   MA1tipo=MODE_CLOSE;//Aplicar à
input ENUM_TIMEFRAMES      MA1TimeFrame=PERIOD_CURRENT;//TimeFrame da Media

input string               Secao4 = "###############";//### Dias da Semana / Tipo Execução ###
input ENUM_LOGIC           Segunda=1;//Executar as Segundas?
input ENUM_LOGIC           Terca=1;//Executar as Terças?
input ENUM_LOGIC           Quarta=1;//Executar as Quartas?
input ENUM_LOGIC           Quinta=1;//Executar as Quintas?
input ENUM_LOGIC           Sexta=1;//Executar as Sextas?

input string               Secao5 = "###############";//### Realização de Parcial ###
input ENUM_LOGIC           IsParcial=0;//Ativar saída parcial?
input double               PrimeiraParcialVolume=0;//Volume da 1ª saída parcial
input double               SegundaParcialVolume=0;//Volume da 2ª saída parcial
input double               SegundaParcialInicio=0;//Saida da 2ª parcial (TP + X Pontos)
input double               TerceiraParcialVolume=0;//Volume da 3ª saída parcial
input double               TerceiraParcialInicio=0;//Saida da 3ª parcial (2ª Parcial + X Pontos)

input string               Secao6 = "###############";//### Mover Stop Loss ###
input ENUM_MOVE_SL_PARCIAL MoveSLExecParc=0;//Mover SL ao Executar
input ENUM_MOVE_SL_PARA    MoveSLPara=0;//Mover o SL para
input double               MoveSLSPread=0;//Considerar uma margem de -x pts no SL
input ENUM_MOVE_SL_PARCIAL MoveSLExecParc1=0;//Mover SL ao Executar
input ENUM_MOVE_SL_PARA    MoveSLPara1=0;//Mover o SL para
input double               MoveSLSPread1=0;//Considerar uma margem de -x pts no SL

input string               Secao7 = "###############";//### Break-Even ###
input ENUM_LOGIC           IsBreakEven=0;//Ativar Break-Even?
input int                  BreakEven=0;//Valor do break-even, zero é o ponto inicial em ticks
input int                  BreakEvenInicio=0;//Valor de inicio em ticks

input string               Secao8 = "###############";//### Financeiro ###
input ENUM_LOGIC           IsGerenciamentoFinanceiro=0;//Ativar Gerenciamento Financeiro?
input double               MaximoLucroDiario=0; //Lucro máximo no dia
input double               MaximoPrejuizoDiario=0; //Prejuízo máximo no dia



//+------------------------------------------------------------------+
//| VARIAVEIS                                                        |
//+------------------------------------------------------------------+
int Len = StringLen(_Symbol);
string SymbolSuffix = StringSubstr(_Symbol, 0, 3); // Obtem o SUFIXO do ativo (WIN, WDO, DOL, IND)
string proxima_barra = "00:00";

MqlTick _price;
MqlRates _rates[];
Logger _logger;
bool _isNewDay;
CPositionInfo _positionInfo;
CTrade _trade;
bool _isBusy;
bool _isNewCandle;
bool _canRewrite;
string _lastTextValidate;
bool _isInverted;

//Grafico
color _corBuy;
color _corSell;
bool _isDesenhar;

//Trailing Stop
bool _isTrailingStop;
int _trailingStopInicio;
int _trailingStop;

//Break Even
bool _isBreakEven;
bool _isBreakEvenExecuted;
int _breakEvenInicio;
int _breakEven;

//Stop no candle anterior
bool _isStopOnLastCandle;
int _StopNoCandle;
bool _waitBreakEvenExecuted;
bool _isPeriodCustom;
int _spreadStopOnLastCandle;
ENUM_TIMEFRAMES _periodStopOnLastCandle;
MqlRates _ratesLasCandle[];

//Period
MqlDateTime _timeCurrent;
MqlDateTime _horaInicio;
MqlDateTime _horaFim;
MqlDateTime _horaInicioIntervalo;
MqlDateTime _horaFimIntervalo;

//Period Interval
string _horaInicioString;
string _horaFimString;
string _horaInicioIntervaloString;
string _horaFimIntervaloString;

double _stopGain;
double _stopLoss;

// Parciais
bool _isParcial;
bool _isPrimeiraParcialExecuted;
double _primeiraParcialVolume;
int _primeiraParcialInicio;
bool _isSegundaParcialExecuted;
double _segundaParcialVolume;
int _segundaParcialInicio;
bool _isTerceiraParcialExecuted;
double _terceiraParcialVolume;
int _terceiraParcialInicio;
string _objNamePrimeiraParcial = "PRIMEIRA_PARCIAL";
string _objNameSegundaParcial = "SEGUNDA_PARCIAL";
string _objNameTerceiraParcial = "TERCEIRA_PARCIAL";

//Gerenciamento Financeiro
double _totalProfitMoney;
double _totalStopLossMoney;
int _totalOrdensVolume;
int _totalOrdensAcertos;

// Total do bot na semana corrente
double _totalProfitMoney_sem;
double _totalStopLossMoney_sem;
int _totalOrdensVolume_sem;
int _totalOrdensAcertos_sem;

// Total do bot no mes corrente
double _totalProfitMoney_mes;
double _totalStopLossMoney_mes;
int _totalOrdensVolume_mes;
int _totalOrdensAcertos_mes;

// Total do bot
double _totalProfitMoney_bot;
double _totalStopLossMoney_bot;
int _totalOrdensVolume_bot;
int _totalOrdensAcertos_bot;

// Minutos
string ZeroMinIni;
string ZeroMinFim;
string ZeroMinIntIni;
string ZeroMinIntFim;

// Travas
bool _JaEntrou;
datetime LiberaProximaOrdem;
int TotalEXEC;

// Indicadores > Media Movel 1
int _MA1Handle;
double _MA1[];


//+------------------------------------------------------------------+
//| Expert initialization function                                   |
//+------------------------------------------------------------------+
int OnInit()
  {

// SL da 1ª parcial só pode ser movido para o break even
   if(MoveSLExecParc == ENUM_MOVE_SL_PARCIAL_PARC1)
     {

      if(MoveSLPara == ENUM_MOVE_SL_PARA_DESABILITADO)
        {
         Alert(__FUNCTION__,"Após executar a 1ª parcial, mova o SL ou revise o SET!");
         return INIT_FAILED;
        }

      if(MoveSLPara != ENUM_MOVE_SL_PARA_BREAK)
        {
         Alert(__FUNCTION__,"Após executar a 1ª parcial, não é possivel mover o SL para a 1ª parcial");
         return INIT_FAILED;
        }

     }


// SL da 2ª parcial só pode ser movido para o break even ou 1ª parcial
   if(MoveSLExecParc1 == ENUM_MOVE_SL_PARCIAL_PARC2)
     {

      if(MoveSLPara1 == ENUM_MOVE_SL_PARA_DESABILITADO)
        {
         Alert(__FUNCTION__,"Após executar a 2ª parcial, mova o SL ou revise o SET!");
         return INIT_FAILED;
        }

     }


   if(DistMAMin >= DistMAMax)
     {
      Alert(__FUNCTION__,"Distancia Minima não pode ser maior ou igual a Distancia Máxima");
      return INIT_FAILED;
     }


// Horario Estrategia
   TimeToStruct(StringToTime(HORARIOS(HoraInicio)),_horaInicio);
   TimeToStruct(StringToTime(HORARIOS(HoraFim)),_horaFim);
   TimeToStruct(StringToTime(HORARIOS(HoraInicioIntervalo)),_horaInicioIntervalo);
   TimeToStruct(StringToTime(HORARIOS(HoraFimIntervalo)),_horaFimIntervalo);

// Horario Estrategia
   if(_horaInicio.hour > _horaFim.hour || (_horaInicio.hour == _horaFim.hour && _horaInicio.min > _horaFim.min))
     {
      Alert(__FUNCTION__," Parâmetros de Horário inválidos!");
      printf("Parâmetros de Horário inválidos!");
      return INIT_FAILED;
     }

// Horario de intervalo da Estrategia
   if(_horaInicioIntervalo.hour > _horaFimIntervalo.hour || (_horaInicioIntervalo.hour == _horaFimIntervalo.hour && _horaInicioIntervalo.min > _horaFimIntervalo.min))
     {
      Alert(__FUNCTION__," Parâmetros de Horário de intervalo inválidos!");
      printf("Parâmetros de Horário de intervalo inválidos!");
      return INIT_FAILED;
     }


// Parciais
   SetIsParcial(IsParcial);
   SetPrimeiraParcialVolume(PrimeiraParcialVolume);
   if(_isParcial)
     {
      SetPrimeiraParcialInicio(StopGainEmTicks);
     }
   SetSegundaParcialVolume(SegundaParcialVolume);
   SetTerceiraParcialVolume(TerceiraParcialVolume);


//Break-Even
   SetIsBreakEven(IsBreakEven);
   SetBreakEvenInicio(BreakEvenInicio);
   SetBreakEven(BreakEven);




   SetNumberMagic(NumeroMagico);

   EventSetMillisecondTimer(1000);
   ManageDealsProfit();

// Grafico de estatisticas
   if(GetRunGraph() && IsStats)
      Box_Creation();


   int initCode = InitIndicators();

   if(initCode != INIT_SUCCEEDED)
     {
      return initCode;
     }
   else
     {
      return INIT_SUCCEEDED;
     }

  }

//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+
void OnDeinit(const int reason)
  {
   if(reason != REASON_CHARTCHANGE)
     {
      ClearDrawParciais();
      ClearDrawStats();

      Comment("");
      EventKillTimer();

     }

  }

//+------------------------------------------------------------------+
//| Expert tick function                                             |
//+------------------------------------------------------------------+
void OnTick()
  {

// Melhora a velocidade do teste no tick a tick
   MqlDateTime stm;
   datetime Datetime = TimeCurrent();
   TimeToStruct(Datetime, stm);

   if(!GetRunGraph() &&
      stm.hour < _horaInicio.hour || (stm.hour <= _horaInicio.hour && stm.min <= _horaInicio.min) ||
      (stm.hour >= _horaFim.hour && stm.min > _horaFim.min) || stm.hour > _horaFim.hour
      )
     {
      return;
     }




   if(!SymbolInfoTick(_Symbol, _price))
     {
      Alert("Erro ao obter a última cotação de preço: ", GetLastError());
      return;
     }

   if(!GetBuffersIndicators())
     {
      Alert("Erro ao obter indicadores!");
      return;
     }


   SetNewday();
   GetBuffers();
   ManageDealsProfit();

   if(HasPositionOpen())
     {

      double _lastprice = NormalizeDouble(GetLastPrice(), 0);

      // Fecha posição encostar na media
      if(IsPositionTypeSell() && _lastprice <= NormalizeDouble(_MA1[0], 0) && FechaPosMA == ENUM_LOGIC_SIM)
         _trade.PositionClose(_Symbol);

      if(IsPositionTypeBuy() && _lastprice >= NormalizeDouble(_MA1[0], 0) && FechaPosMA == ENUM_LOGIC_SIM)
         _trade.PositionClose(_Symbol);

      ManagePosition();
      return;
     }
   else
     {
      ResetPosition();
     }

   ClearObjectAutoTrading();
   OnTickHandler();

  }




//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+
void OnTimer()
  {

   if(GetRunGraph())
     {

      OnDragParcial();
      ManageDrawParcial();

      if(IsStats)
         Box_Creation();

      ChartRedraw();
     }

  }

//+------------------------------------------------------------------+
//| Tester function                                                  |
//+------------------------------------------------------------------+
double OnTester()
  {

   /*
    double ticksize = SymbolInfoDouble(_Symbol,SYMBOL_TRADE_TICK_SIZE);
    double tickvalue = SymbolInfoDouble(_Symbol,SYMBOL_TRADE_TICK_VALUE);
    double blinicial =(TesterStatistics(STAT_INITIAL_DEPOSIT)/ticksize)*tickvalue;
    double dd = ((TesterStatistics(STAT_BALANCE_DD)/ticksize)*tickvalue)/blinicial;

    double profitfactor = TesterStatistics(STAT_PROFIT_FACTOR);
    double lucro = ((TesterStatistics(STAT_PROFIT)/ticksize)*tickvalue)/blinicial;
    double taxadeacerto = TesterStatistics(STAT_PROFIT_TRADES)/TesterStatistics(STAT_TRADES);
    _logger.Log((profitfactor + taxadeacerto)/2);
    double ret=(((lucro*0.45)+(dd*0.15)+(((profitfactor + taxadeacerto)/2)*0.40))/3)*lr_corr();

    return(ret);
    */

   return 0;

  }
//+------------------------------------------------------------------+



// Lucro corretora
double lr_corr()
  {
//--- object to access data on deals
   CDealInfo     deal;
//--- object for storing profit/loss of each deal
   CArrayDouble *profit=new CArrayDouble;
//--- objects for storing the balance
   CArrayDouble *balance_total=new CArrayDouble;
//--- initial balance
   double        balance=0;
//--- receive history
   HistorySelect(0,TimeCurrent());
//--- total amount of deals
   int deals_total=HistoryDealsTotal();
//--- receive data on deals' profit and balance
   for(int i=0; i<deals_total; i++)
     {
      //--- move to the deal having i index
      deal.SelectByIndex(i);
      //--- receive initial balance
      if(deal.DealType()==DEAL_TYPE_BALANCE)
        {
         if(NormalizeDouble(deal.Profit()+deal.Swap(),2)>=0.0)
            if(balance==0.0)
               balance=deal.Profit();
        }
      //--- receive profit and balance
      if(deal.DealType()==DEAL_TYPE_BUY || deal.DealType()==DEAL_TYPE_SELL)
         if(deal.Entry()==DEAL_ENTRY_OUT || deal.Entry()==DEAL_ENTRY_INOUT)
           {
            profit.Add(NormalizeDouble(deal.Profit()+deal.Swap()+deal.Commission(),2));
            balance_total.Add(balance);
            balance=balance+NormalizeDouble(deal.Profit()+deal.Swap()+deal.Commission(),2);
           }
     }
   balance_total.Add(balance_total.At(balance_total.Total()-1)+profit.At(balance_total.Total()-1));
//--- copy data on the balance to double type array
   double arr_balance[];
   ArrayResize(arr_balance,balance_total.Total());
   for(int i=0; i<balance_total.Total(); i++)
      arr_balance[i]=balance_total.At(i);
//--- copy data on profit to double type array
   double arr_profit[];
   ArrayResize(arr_profit,profit.Total());
   for(int i=0; i<profit.Total(); i++)
      arr_profit[i]=profit.At(i);
//--- linear regression
//--- number of independent variables
   int nvars=1;
//--- sample volume
   int npoints=balance_total.Total();
//--- create the matrix of parameters for the linear regression
   CMatrixDouble xy(npoints,nvars+1);
   for(int i=0; i<npoints; i++)
     {
      xy[i].Set(0,i);
      xy[i].Set(1,arr_balance[i]);
     }
//--- variable for detecting calculations result (successful, unsuccessful)
   int info;
//--- class objects necessary for storing data on calculations
   CLinearModelShell lm;
   CLRReportShell    ar;
//--- arrays for storing regression results
   double lr_coeff[];
   double lr_values[];
   ArrayResize(lr_values,npoints);
//--- calculate linear regression ratios
   CAlglib::LRBuild(xy,npoints,nvars,info,lm,ar);
//--- receive linear regression ratios
   CAlglib::LRUnpack(lm,lr_coeff,nvars);
//--- receive recovered linear regression values
   for(int i=0; i<npoints; i++)
      lr_values[i]=lr_coeff[0]*i+lr_coeff[1];
//--- calculate Expected Payoff
   double exp_payoff,tmp1,tmp2,tmp3;
   CAlglib::SampleMoments(arr_profit,exp_payoff,tmp1,tmp2,tmp3);
//--- calculate HPR array
   double HPR[];
//   ArrayResize(HPR,balance_total.Total()-1);
//   for(int i=0;i<balance_total.Total()-1;i++)
//      HPR[i]=balance_total.At(i+1)/balance_total.At(i);
////--- calculate standard deviation and math expectation from HPR
//   double AHPR,SD;
//   CAlglib::SampleMoments(HPR,AHPR,SD,tmp2,tmp3);
//   SD=MathSqrt(SD);
//--- calculate LR Correlation
   double lr_corr=CAlglib::PearsonCorr2(arr_balance,lr_values);
//--- receive LR Standard Error
   double lr_stand_err=0;
   for(int i=0; i<npoints; i++)
     {
      double delta=MathAbs(arr_balance[i]-lr_values[i]);
      lr_stand_err=lr_stand_err+delta*delta;
     }
   lr_stand_err=MathSqrt(lr_stand_err/(npoints-2));
//--- calculate Sharpe Ratio
//double sharpe_ratio=(AHPR-1)/SD;
//--- delete objects
   delete profit;
   delete balance_total;
   return lr_corr;
  }


//+------------------------------------------------------------------+
//|  Inicializa indicadores                                          |
//+------------------------------------------------------------------+
int InitIndicators()
  {

   if(InitMA1() != INIT_SUCCEEDED)
      return INIT_FAILED;

   return INIT_SUCCEEDED;

  }

//+------------------------------------------------------------------+
//|  Inicializa Indicador > MEDIA MOVEL 1                            |
//+------------------------------------------------------------------+
int InitMA1()
  {

   _MA1Handle = iMA(_Symbol, MA1TimeFrame, MA1Periodo, MA1Deslocar, MA1Metodo, MA1tipo);

   if(_MA1Handle < 0)
     {
      Alert("Erro ao criar indicador Media Movel("+MA1Periodo+"): ", GetLastError());
      return INIT_FAILED;
     }

   return INIT_SUCCEEDED;
  }

//+------------------------------------------------------------------+
//| ATUALIZA OS INDICADORES                                          |
//+------------------------------------------------------------------+
bool GetBuffersIndicators()
  {

   if(!GetBufferMA1())
      return false;

   return true;

  }

//+------------------------------------------------------------------+
//|  Inicializa Indicador > EMA1                                     |
//+------------------------------------------------------------------+
bool GetBufferMA1()
  {

   ZeroMemory(_MA1);
   ArraySetAsSeries(_MA1, true);
   ArrayFree(_MA1);

   int copiedBuffer = CopyBuffer(_MA1Handle, 0, 0, 15, _MA1);

   return copiedBuffer > 0;

  }



// Novo Dia
void SetNewday()
  {
   static int oldDay;
   int newDay = GetTimeCurrent().day;
   bool isNewDay = false;

   if(oldDay != newDay)
     {

      isNewDay = true;
      oldDay = newDay;
      _logger.Log("Robo: " + robot_name +" (" + SymbolSuffix +")");


      _JaEntrou = false; // Reseta a ordem do dia
      TotalEXEC = 0;

     }

   _isNewDay = isNewDay;

  }


// Existe posição aberta?
bool HasPositionOpen()
  {
   return _positionInfo.Select(_Symbol) && GetPositionMagicNumber() == NumeroMagico;
  }

// Gerenciamento das posições abertas
void ManagePosition()
  {
   if(_isBusy)
      return;

   _isBusy = true;

   if(GetPositionMagicNumber() != NumeroMagico)
     {
      return;
     }

   MqlDateTime stm;
   datetime Datetime = TimeCurrent();
   TimeToStruct(Datetime, stm);

   if(FecharPosition && stm.hour >= _horaFim.hour && stm.min >= _horaFim.min)
     {
      ClosePosition();
     }

   /*
      if(!HasPositionLossOrPositionGain())
        {
         //RepositionTrade();
         ResetPosition();
        }
      else
        {
   */
   ManageStopOnLastCandle();
   ManageTrailingStop();
   ManageBreakEven();
   ManageParcial();
//     }

   _isBusy = false;

  }

// Obtem o tempo atual
MqlDateTime GetTimeCurrent()
  {
   TimeCurrent(_timeCurrent);
   return _timeCurrent;
  }

// Obtem as posições pelo Magic Number
int GetPositionMagicNumber()
  {
   return (int)PositionGetInteger(POSITION_MAGIC);
  }

// SET Hora de inicio
void SetHoraInicio(string hora)
  {
   _horaInicioString = hora;
   TimeToStruct(StringToTime("1990.04.02 " + hora), _horaInicio);
  };

// GET Hora de inicio
MqlDateTime GetHoraInicio()
  {
   return _horaInicio;
  };

// SET Hora de termino
void SetHoraFim(string hora)
  {
   _horaFimString = hora;
   TimeToStruct(StringToTime("1990.04.02 " + hora), _horaFim);
  };

// GET Hora de termino
MqlDateTime GetHoraFim()
  {
   return _horaFim;
  };

// SET hora Inicio Intervalo
void SetHoraInicioIntervalo(string hora)
  {
   _horaInicioIntervaloString = hora;
   TimeToStruct(StringToTime("1990.04.02 " + hora), _horaInicioIntervalo);
  };

// GET hora Inicio Intervalo
MqlDateTime GetHoraInicioIntervalo()
  {
   return _horaInicioIntervalo;
  };

// SET ora Fim Intervalo
void SetHoraFimIntervalo(string hora)
  {
   _horaFimIntervaloString = hora;
   TimeToStruct(StringToTime("1990.04.02 " + hora), _horaFimIntervalo);
  };

// GET hora Fim Intervalo
MqlDateTime GetHoraFimIntervalo()
  {
   return _horaFimIntervalo;
  };

// Fechar posição
void ClosePosition()
  {
   if(!HasPositionOpen())
     {
      _logger.Log("Não existe posição em aberto");
      return;
     }

   _trade.PositionClose(_Symbol);

// Apaga ordens pendentes
   ApagaOrdem();

   _logger.Log("Posição zerada!");

  }

// Esta Ganhando ou perdendo
bool HasPositionLossOrPositionGain()
  {
   return GetPositionLoss() > 0.0 && GetPositionGain() > 0.0;
  }

// Apos Executar algum contrato move o SL
bool RepositionTrade()
  {
   if(!HasPositionOpen())
      return false;

   double price = GetPositionPriceOpen();

   if(IsPositionTypeBuy())
     {
      double stopGain = NormalizeDouble((price + _stopGain), _Digits);
      double stopLoss = NormalizeDouble((price - _stopLoss), _Digits);

      _trade.PositionModify(_Symbol, stopLoss, stopGain);
     }

   if(IsPositionTypeSell())
     {
      double stopGain = NormalizeDouble((price - _stopGain), _Digits);
      double stopLoss = NormalizeDouble((price + _stopLoss), _Digits);

      _trade.PositionModify(_Symbol, stopLoss, stopGain);
     }

   _logger.Log("Stop, Gain e gerenciamento retomado");

   return true;

  }

// Reorganiza a parciais
void ResetPosition()
  {

   if(GetRunGraph())
     {
      ClearDraw(_objNamePrimeiraParcial);
      ClearDraw(_objNameSegundaParcial);
      ClearDraw(_objNameTerceiraParcial);
     }

   /*
      if(HasPositionOpen())
         return;
   */

   _isPrimeiraParcialExecuted = false;
   _isSegundaParcialExecuted = false;
   _isTerceiraParcialExecuted = false;
   _isBreakEvenExecuted = false;
  }

// StopLoss no ultimo Candle
bool ManageStopOnLastCandle()
  {

   if(_isBreakEven && _waitBreakEvenExecuted)
     {
      if(!_isBreakEvenExecuted)
        {
         return false;
        }
     }

   if(!_isStopOnLastCandle || !_isNewCandle)
     {
      return false;
     }


   if(CopyRates(_Symbol, _isPeriodCustom ? _periodStopOnLastCandle : Periodo, 0, (_StopNoCandle), _ratesLasCandle) <= 0)
     {
      return false;
     }

//Posicao menor é o mais longe, ou seja, _rates[0] é o primeiro e _rates[1] é o ultimo
   MqlRates _candleAnterior = _ratesLasCandle[0];

   if(IsPositionTypeBuy())
     {
      if(GetPositionLoss() < _candleAnterior.low - ToPoints(GetSpreadStopOnLastCandle()))
        {
         _trade.PositionModify(_Symbol, GetStopLoss_Symbol(_candleAnterior.low - (GetSpreadStopOnLastCandle() * 2)), GetPositionGain());
         _logger.Log("=> StopLoss ajustado para: " + (string)GetPositionLoss());
         return true;
        }
     }

   if(IsPositionTypeSell())
     {
      if(GetPositionLoss() > _candleAnterior.high + ToPoints(GetSpreadStopOnLastCandle()))
        {
         _trade.PositionModify(_Symbol, GetStopLoss_Symbol(_candleAnterior.high + (GetSpreadStopOnLastCandle() * 2)), GetPositionGain());
         _logger.Log("=> StopLoss ajustado para: " + (string)GetPositionLoss());
         return true;
        }
     }

   return false;

  }

// Ajusta Trailingstop
bool ManageTrailingStop()
  {
   if(!_isTrailingStop)
     {
      return false;
     }

   if(IsPositionTypeBuy())
     {

      if(GetLastPrice() - GetPositionLoss() >= _stopLoss + ToPoints(_trailingStopInicio))
        {
         _trade.PositionModify(_Symbol, GetPositionLoss() + ToPoints(_trailingStop), GetPositionGain());
         _logger.Log("=> Stop ajustado para o trailing stop. " + (string)GetPositionLoss());
         return true;
        }

     }

   if(IsPositionTypeSell())
     {

      if(GetPositionLoss() - GetLastPrice() >= _stopLoss + ToPoints(_trailingStopInicio))
        {
         _trade.PositionModify(_Symbol, GetPositionLoss() - ToPoints(_trailingStop), GetPositionGain());
         _logger.Log("=> Stop ajustado para o  trailing stop. " + (string)GetPositionLoss());
         return true;
        }

     }

   return false;

  }

// Ajustar breakEven
bool ManageBreakEven()
  {
   if(!_isBreakEven || _isBreakEvenExecuted)
     {
      return false;
     }

   if(IsPositionTypeBuy())
     {

      if(GetLastPrice() >= GetPositionPriceOpen() + ToPoints(_breakEvenInicio) && GetPositionLoss() < GetPositionPriceOpen())
        {
         _trade.PositionModify(_Symbol, GetPositionPriceOpen() + ToPoints(_breakEven), GetPositionGain());
         _logger.Log("=> Stop ajustado para o  break even: " + (string)(GetPositionPriceOpen() + _breakEven));
         _isBreakEvenExecuted = true;
        }
     }

   if(IsPositionTypeSell())
     {

      if(GetLastPrice() <= GetPositionPriceOpen() - ToPoints(_breakEvenInicio) && GetPositionLoss() > GetPositionPriceOpen())
        {
         _trade.PositionModify(_Symbol, GetPositionPriceOpen() - ToPoints(_breakEven), GetPositionGain());
         _logger.Log("=> Stop ajustado para o break even: " + (string)(GetPositionPriceOpen() - _breakEven));
         _isBreakEvenExecuted = true;
        }
     }

   return _isBreakEvenExecuted;

  }

// Gerenciamento das Parciais
bool ManageParcial()
  {

   if(!_isParcial)
     {
      return false;
     }

   if(GetLastPrice() <= 0)
      return false;


// Desenha linha das parciais
   if(GetRunGraph())
     {
      ManageDrawParcial();
     }


// 1ª Parcial - Saida de compra
   if(IsPositionTypeBuy() && !_isPrimeiraParcialExecuted && GetLastPrice() >= _primeiraParcialInicio)
     {
      return ExecutePrimeiraParcial();
     }

// 2ª Parcial - Saida de compra
   if(IsPositionTypeBuy() && !_isSegundaParcialExecuted && GetLastPrice() >= _segundaParcialInicio && SegundaParcialVolume >= 1)
     {
      return ExecuteSegundaParcial();
     }
// 3ª Parcial - Saida de compra
   if(IsPositionTypeBuy() && !_isTerceiraParcialExecuted && GetLastPrice() >= _terceiraParcialInicio && TerceiraParcialVolume >= 1)
     {
      return ExecuteTerceiraParcial();
     }






// 1ª Parcial - Saida de Venda
   if(IsPositionTypeSell() && !_isPrimeiraParcialExecuted && GetLastPrice() <= _primeiraParcialInicio)
     {
      return ExecutePrimeiraParcial();
     }

// 2ª Parcial - Saida de Venda
   if(IsPositionTypeSell() && !_isSegundaParcialExecuted && GetLastPrice() <= _segundaParcialInicio && SegundaParcialVolume >= 1)
     {
      return ExecuteSegundaParcial();
     }
// 3ª Parcial - Saida de Venda
   if(IsPositionTypeSell() && !_isTerceiraParcialExecuted && GetLastPrice() <= _terceiraParcialInicio && TerceiraParcialVolume >= 1)
     {
      return ExecuteTerceiraParcial();
     }

   return false;

  }

// Obtem o Loss da posição
double GetPositionLoss()
  {
   return PositionGetDouble(POSITION_SL);
  }

// Obtem o Gain da posição
double GetPositionGain()
  {
   return PositionGetDouble(POSITION_TP);
  }

// Preço de abertura da posição
double GetPositionPriceOpen()
  {
   return NormalizeDouble(PositionGetDouble(POSITION_PRICE_OPEN), _Digits);
  }

// Foi uma compra?
bool IsPositionTypeBuy()
  {
   return GetPositionType() == POSITION_TYPE_BUY;
  }

// Foi uma venda?
bool IsPositionTypeSell()
  {
   return GetPositionType() == POSITION_TYPE_SELL;
  }

// Tipo da Posição
int GetPositionType()
  {
   return (int)PositionGetInteger(POSITION_TYPE);
  }

// Retorno da posição
double GetPositionProfit()
  {
   return PositionGetDouble(POSITION_PROFIT);
  }

// Ticks pra ponto
double ToPoints(int tickValue)
  {
   return tickValue * GetSymbolTickSize();
  }

// Pontos pra Ticks
double TicksPonto(int tickValue)
  {
   return tickValue / GetSymbolTickSize();
  }

// Symbol Tick Size
double GetSymbolTickSize()
  {
   return SymbolInfoDouble(_Symbol, SYMBOL_TRADE_TICK_SIZE);
  }

// Ultimo Preço
double GetLastPrice()
  {
   if(TipoUltimoPreco == ENUM_LAST_PRICE_TYPE_LAST)
     {
      return _price.last;
     }

   if(TipoUltimoPreco == ENUM_LAST_PRICE_TYPE_ASK_OR_BID)
     {
      return (IsPositionTypeSell() ? _price.ask : _price.bid);
     }

   return NormalizeDouble(0, _Digits);

  }

// Valor da Parcila
double GetParcialValue(string objName)
  {
   return NormalizeDouble(ObjectGetDouble(0, objName, OBJPROP_PRICE), _Digits);
  }

// Apaga Objetos da tela
void ClearDraw(string objName)
  {
   ObjectDelete(0, objName);
  }


// Desenha obejtos
void Draw(double price, color cor)
  {

   if(!_isDesenhar)
     {
      return;
     }

   string objName = "LINE" + (string)price;
   ObjectCreate(0, objName, OBJ_HLINE, 0, 0, price);

   ObjectSetInteger(0, objName, OBJPROP_COLOR, cor);
   ObjectSetInteger(0, objName, OBJPROP_BORDER_COLOR, clrBlack);
   ObjectSetInteger(0, objName, OBJPROP_STYLE, STYLE_SOLID);
   ObjectSetInteger(0, objName, OBJPROP_WIDTH, 1);
   ObjectSetInteger(0, objName, OBJPROP_BACK, true);
   ObjectSetInteger(0, objName, OBJPROP_FILL, true);

//ARROW PRICE
   objName = "SETA" + (string)price;
   ObjectCreate(0, objName, OBJ_ARROW_RIGHT_PRICE, 0, _rates[0].time, price);
   ObjectSetInteger(0, objName, OBJPROP_COLOR, cor);
   ObjectSetInteger(0, objName, OBJPROP_BORDER_COLOR, clrBlack);
   ObjectSetInteger(0, objName, OBJPROP_STYLE, STYLE_SOLID);
   ObjectSetInteger(0, objName, OBJPROP_WIDTH, 2);
   ObjectSetInteger(0, objName, OBJPROP_BACK, false);
   ObjectSetString(0, objName, OBJPROP_TEXT, "ENTRADA EM " + (string)price);
  }

// Novo Candle
void SetNewCandle()
  {

   static datetime OldTime;
   datetime NewTime[1];
   bool newBar = false;

   int copied = CopyTime(Symbol(), Periodo, 0, 1, NewTime);

   if(copied > 0 && OldTime != NewTime[0])
     {
      newBar = true;
      OldTime = NewTime[0];
     }

   _isNewCandle = newBar;

  }

// Executa Primeira Parcial
bool ExecutePrimeiraParcial()
  {
   if(_isPrimeiraParcialExecuted)
      return false;

   if(!HasPositionOpen())
     {
      if(IsDebug)
         _logger.Log("Não existe posição em aberto");
      return false;
     }

   if(GetPositionVolume() < _primeiraParcialVolume)
     {
      if(IsDebug)
         _logger.Log("Volume da parcial maior do que posição atual");
      return false;
     }

   if(IsHedging())
     {
      _trade.PositionClosePartial(_Symbol, _primeiraParcialVolume);
     }
   else
     {
      if(IsPositionTypeBuy())
        {
         _trade.Sell(_primeiraParcialVolume, _Symbol);
        }
      if(IsPositionTypeSell())
        {
         _trade.Buy(_primeiraParcialVolume, _Symbol);
        }
     }
   _isPrimeiraParcialExecuted = true;

   if(IsDebug)
      _logger.Log("Saída 1ª parcial em " + (string)GetLastPrice() + " com volume " + (string)_primeiraParcialVolume);



   if(MoveSLExecParc == ENUM_MOVE_SL_PARCIAL_PARC1)
     {

      double SL_MOVE;
      double price = GetPositionPriceOpen();

      if(IsPositionTypeSell())
        {

         if(MoveSLPara == ENUM_MOVE_SL_PARA_BREAK)
            SL_MOVE = GetStopLoss_Symbol(GetPositionPriceOpen() + MoveSLSPread);

         if(IsDebug)
            _logger.Log("SL AJUSTADO PARA: " + SL_MOVE);
         _trade.PositionModify(_Symbol, SL_MOVE, GetStopLoss_Symbol(price - _stopGain));

        }
      if(IsPositionTypeBuy())
        {

         if(MoveSLPara == ENUM_MOVE_SL_PARA_BREAK)
            SL_MOVE = GetStopLoss_Symbol(GetPositionPriceOpen() - MoveSLSPread);

         if(IsDebug)
            _logger.Log("SL AJUSTADO PARA: " + SL_MOVE);
         _trade.PositionModify(_Symbol, SL_MOVE, GetStopLoss_Symbol(price + _stopGain));
         //_trade.PositionModify(_Symbol, GetStopLoss_Symbol(_primeiraParcialInicio - StopGainEmTicks + GetSymbolTickSize()), GetStopLoss_Symbol(_terceiraParcialInicio));
        }


     }



   return true;

  }

// Executa Segunda Parcial
bool ExecuteSegundaParcial()
  {
   if(_isSegundaParcialExecuted)
      return false;

   if(!HasPositionOpen())
     {
      if(IsDebug)
         _logger.Log("Não existe posição em aberto");
      return false;
     }

   if(GetPositionVolume() < _segundaParcialVolume)
     {
      if(IsDebug)
         _logger.Log("Volume da parcial maior do que posição atual");
      return false;
     }

   if(IsHedging())
     {
      _trade.PositionClosePartial(_Symbol, _segundaParcialVolume);
     }
   else
     {
      if(IsPositionTypeBuy())
        {
         _trade.Sell(_segundaParcialVolume, _Symbol);
        }
      if(IsPositionTypeSell())
        {
         _trade.Buy(_segundaParcialVolume, _Symbol);
        }
     }
   _isSegundaParcialExecuted = true;

   if(IsDebug)
      _logger.Log("Saída 2ª parcial em " + (string)GetLastPrice() + " com volume " + (string)_segundaParcialVolume);


   if(MoveSLExecParc == ENUM_MOVE_SL_PARCIAL_PARC2 || MoveSLExecParc1 == ENUM_MOVE_SL_PARCIAL_PARC2)
     {

      double SL_MOVE;
      double price = GetPositionPriceOpen();

      if(IsPositionTypeSell())
        {

         // 1ª parcial
         if(MoveSLPara == ENUM_MOVE_SL_PARA_BREAK)
            SL_MOVE = GetStopLoss_Symbol(_primeiraParcialInicio + StopGainEmTicks + MoveSLSPread);

         // 2ª parcial
         if(MoveSLPara1 == ENUM_MOVE_SL_PARA_BREAK)
            SL_MOVE = GetStopLoss_Symbol(_primeiraParcialInicio + StopGainEmTicks + MoveSLSPread1);

         // 1ª parcial
         if(MoveSLPara == ENUM_MOVE_SL_PARA_PARC1)
            SL_MOVE = GetStopLoss_Symbol(_primeiraParcialInicio + MoveSLSPread);

         // 2ª parcial
         if(MoveSLPara1 == ENUM_MOVE_SL_PARA_PARC1)
            SL_MOVE = GetStopLoss_Symbol(_primeiraParcialInicio + MoveSLSPread1);

         if(IsDebug)
            _logger.Log("SL AJUSTADO PARA: " + SL_MOVE);
         _trade.PositionModify(_Symbol, SL_MOVE, GetStopLoss_Symbol(price - _stopGain));

        }
      if(IsPositionTypeBuy())
        {
         // 1ª parcial
         if(MoveSLPara == ENUM_MOVE_SL_PARA_BREAK)
            SL_MOVE = GetStopLoss_Symbol(_primeiraParcialInicio - StopGainEmTicks - MoveSLSPread);

         // 2ª parcial
         if(MoveSLPara1 == ENUM_MOVE_SL_PARA_BREAK)
            SL_MOVE = GetStopLoss_Symbol(_primeiraParcialInicio - StopGainEmTicks - MoveSLSPread1);

         // 1ª parcial
         if(MoveSLPara == ENUM_MOVE_SL_PARA_PARC1)
            SL_MOVE = GetStopLoss_Symbol(_primeiraParcialInicio - MoveSLSPread);

         // 2ª parcial
         if(MoveSLPara1 == ENUM_MOVE_SL_PARA_PARC1)
            SL_MOVE = GetStopLoss_Symbol(_primeiraParcialInicio - MoveSLSPread1);

         if(IsDebug)
            _logger.Log("SL AJUSTADO PARA: " + SL_MOVE);
         _trade.PositionModify(_Symbol, SL_MOVE, GetStopLoss_Symbol(price + _stopGain));
         //_trade.PositionModify(_Symbol, GetStopLoss_Symbol(_primeiraParcialInicio - StopGainEmTicks + GetSymbolTickSize()), GetStopLoss_Symbol(_terceiraParcialInicio));
        }


     }


   return true;

  }

// Executa Terceira parcial
bool ExecuteTerceiraParcial()
  {
   if(_isTerceiraParcialExecuted)
      return false;

   if(!HasPositionOpen())
     {
      if(IsDebug)
         _logger.Log("Não existe posição em aberto");
      return false;
     }

   if(GetPositionVolume() < _terceiraParcialVolume)
     {
      if(IsDebug)
         _logger.Log("Volume da parcial maior do que posição atual");
      return false;
     }



   if(IsHedging())
     {
      _trade.PositionClosePartial(_Symbol, _terceiraParcialVolume);
     }
   else
     {
      if(IsPositionTypeBuy())
        {
         _trade.Sell(_terceiraParcialVolume, _Symbol);
        }
      if(IsPositionTypeSell())
        {
         _trade.Buy(_terceiraParcialVolume, _Symbol);
        }
     }
   _isTerceiraParcialExecuted = true;

   if(IsDebug)
      _logger.Log("Saída 3ª parcial em " + (string)GetLastPrice() + " com volume " + (string)_terceiraParcialVolume);

   return true;

  }

// Stop Spread no ultimo Candle
int GetSpreadStopOnLastCandle()
  {
   return _spreadStopOnLastCandle;
  }

// Volume da Posição atual
double GetPositionVolume()
  {
   return PositionGetDouble(POSITION_VOLUME);
  }

// Função para calcular o SL valido de acordo com o Tick Size
double GetStopLoss_Symbol(double pValueToRound)
  {
   double vRound = MathRound(pValueToRound / GetSymbolTickSize()) * GetSymbolTickSize();
   return vRound;
  }

// Apaga todos os objetos desenhados
void ClearObjectAutoTrading()
  {
   /*
    if(!_canRewrite)
       return;
    */

   ObjectsDeleteAll(ChartID(), 0, OBJ_ARROW_BUY);
   ObjectsDeleteAll(ChartID(), 0, OBJ_ARROW_SELL);
   ObjectsDeleteAll(ChartID(), 0, OBJ_TREND);
  }


// Gerenciamento financeiro
void ManageDealsProfit()
  {
   string CurrDate = TimeToString(TimeCurrent(), TIME_DATE);
//HistorySelect(StringToTime(CurrDate), TimeCurrent()); // Somente do dia
   HistorySelect(0, TimeCurrent());

   ulong ticket = 0;
   double price;
   double profit;
   datetime time;
   string symbol;
   string comment;
   long type;
   long entry;
   double volume;
   ulong magic;

// Dia
   double totalGainMoney = 0;
   double totalLossMoney = 0;
   int qtdOrdensVolume = 0;
   int acertos_dia = 0;

// Semana
   double totalGainMoney_sem = 0;
   double totalLossMoney_sem = 0;
   int qtdOrdensVolume_sem = 0;
   int acertos_sem = 0;

// Mes
   datetime TC=TimeCurrent();
   MqlDateTime TCM;
   TimeToStruct(TC,TCM);
   double totalGainMoney_mes = 0;
   double totalLossMoney_mes = 0;
   int qtdOrdensVolume_mes = 0;
   int acertos_mes = 0;


// Total do bot (magic number)
   double totalGainMoney_total = 0;
   double totalLossMoney_total = 0;
   int qtdOrdensVolume_total = 0;
   int acertos_total = 0;




   for(int i = HistoryDealsTotal() - 1; i >= 0; i--)
     {
      ticket = HistoryDealGetTicket(i);

      if(ticket <= 0)
        {
         continue;
        }

      price = HistoryDealGetDouble(ticket, DEAL_PRICE);
      time = (datetime)HistoryDealGetInteger(ticket, DEAL_TIME);
      symbol = HistoryDealGetString(ticket, DEAL_SYMBOL);
      comment = HistoryDealGetString(ticket, DEAL_COMMENT);
      type = HistoryDealGetInteger(ticket, DEAL_TYPE);
      magic = HistoryDealGetInteger(ticket, DEAL_MAGIC);
      entry = HistoryDealGetInteger(ticket, DEAL_ENTRY);
      profit = HistoryDealGetDouble(ticket, DEAL_PROFIT);
      volume = HistoryDealGetDouble(ticket, DEAL_VOLUME);

      MqlDateTime _TimeOrder;
      TimeToStruct(time,_TimeOrder);

      if(symbol != _Symbol)
        {
         continue;
        }

      if(magic != NumeroMagico)
        {
         continue;
        }

      if(!price && !time)
        {
         continue;
        }

      if(profit < 0)
        {

         if(IsStats)
           {

            // Dia
            if(time >= TimeToString(TimeCurrent(),TIME_DATE))
              {
               totalLossMoney += profit;
               qtdOrdensVolume += volume;
               acertos_dia = acertos_dia - 1;
              }

            // Semana (deste mes)
            if(WeekNumber(_TimeOrder) ==  WeekNumber(TCM) && TCM.mon == _TimeOrder.mon)
              {
               totalLossMoney_sem += profit;
               qtdOrdensVolume_sem += volume;
               acertos_sem = acertos_sem - 1;
              }

            // Mes
            if(TCM.mon == _TimeOrder.mon)
              {
               totalLossMoney_mes += profit;
               qtdOrdensVolume_mes += volume;
               acertos_mes = acertos_mes - 1;
              }

            totalLossMoney_total += profit;
            qtdOrdensVolume_total += volume;
            acertos_total = acertos_total - 1;

           }
         else
           {
            totalLossMoney += profit;
            qtdOrdensVolume += volume;
           }

         continue;
        }

      if(profit > 0)
        {

         if(IsStats)
           {
            // Dia
            if(time >= TimeToString(TimeCurrent(),TIME_DATE))
              {
               totalGainMoney += profit;
               qtdOrdensVolume += volume;
               acertos_dia = acertos_dia + 1;
              }

            // Semana (deste mes)
            if(WeekNumber(_TimeOrder) ==  WeekNumber(TCM) && TCM.mon == _TimeOrder.mon)
              {
               totalGainMoney_sem += profit;
               qtdOrdensVolume_sem += volume;
               acertos_sem = acertos_sem + 1;
              }

            // Mes
            if(TCM.mon == _TimeOrder.mon)
              {
               totalGainMoney_mes += profit;
               qtdOrdensVolume_mes += volume;
               acertos_mes = acertos_mes + 1;
              }

            totalGainMoney_total += profit;
            qtdOrdensVolume_total += volume;
            acertos_total = acertos_total + 1;

           }
         else
           {
            totalGainMoney += profit;
            qtdOrdensVolume += volume;
           }

         continue;
        }



     }

   if(IsStats)
     {
      // Total por dia
      _totalProfitMoney = totalGainMoney;
      _totalStopLossMoney = totalLossMoney;
      _totalOrdensVolume = qtdOrdensVolume;
      _totalOrdensAcertos = acertos_dia + ((qtdOrdensVolume - acertos_dia) /2);

      // Semana
      _totalProfitMoney_sem = totalGainMoney_sem;
      _totalStopLossMoney_sem = totalLossMoney_sem;
      _totalOrdensVolume_sem = qtdOrdensVolume_sem;
      _totalOrdensAcertos_sem = acertos_sem + ((qtdOrdensVolume_sem - acertos_sem) /2);

      // Mês
      _totalProfitMoney_mes = totalGainMoney_mes;
      _totalStopLossMoney_mes = totalLossMoney_mes;
      _totalOrdensVolume_mes = qtdOrdensVolume_mes;
      _totalOrdensAcertos_mes = acertos_mes + ((qtdOrdensVolume_mes - acertos_mes) /2);

      // Total do bot
      _totalProfitMoney_bot = totalGainMoney_total;
      _totalStopLossMoney_bot = totalLossMoney_total;
      _totalOrdensVolume_bot = qtdOrdensVolume_total;
      _totalOrdensAcertos_bot = acertos_total + ((qtdOrdensVolume_total - acertos_total) /2);
     }

  }


// Validação para entrar na operação
bool Validate()
  {

   bool isValid = true;
   MqlDateTime time = GetTimeCurrent();
   string TMP_HoraAtual = TimeToString(TimeCurrent(),TIME_MINUTES);
   int replaced;


// NAO RODAR EM DETERMINADO DIA DA SEMANA
   if(!Segunda && (ENUM_DAY_OF_WEEK)time.day_of_week == 1)
     {
      isValid = false;
      if(IsDebug)
         _logger.Log("Não Executar na Segunda Feira!");
     }
   if(!Terca && (ENUM_DAY_OF_WEEK)time.day_of_week == 2)
     {
      isValid = false;
      if(IsDebug)
         _logger.Log("Não Executar na Terça Feira!");
     }
   if(!Quarta && (ENUM_DAY_OF_WEEK)time.day_of_week == 3)
     {
      isValid = false;
      if(IsDebug)
         _logger.Log("Não Executar na Quarta Feira!");
     }
   if(!Quinta && (ENUM_DAY_OF_WEEK)time.day_of_week == 4)
     {
      isValid = false;
      if(IsDebug)
         _logger.Log("Não Executar na Quinta Feira!");
     }
   if(!Sexta && (ENUM_DAY_OF_WEEK)time.day_of_week == 5)
     {
      isValid = false;
      if(IsDebug)
         _logger.Log("Não Executar na Sexta Feira!");
     }


   MqlDateTime stm;
   datetime Datetime = TimeCurrent();
   TimeToStruct(Datetime, stm);

   if(stm.hour < _horaInicio.hour || (stm.hour <= _horaInicio.hour && stm.min <= _horaInicio.min) ||
      (stm.hour >= _horaFim.hour && stm.min > _horaFim.min) || stm.hour > _horaFim.hour)
     {
      isValid = false;
      _logger.Log("Horário somente entre " + _horaInicioString + " e " + _horaFimString);
     }



   if(isValid && (stm.hour >= _horaInicioIntervalo.hour && stm.min >= _horaInicioIntervalo.min &&
                  stm.hour <= _horaFimIntervalo.hour && stm.min <= _horaFimIntervalo.min
                 ))
     {
      isValid = false;
      _logger.Log("Horário deve estar fora do intervalo de " + _horaInicioIntervalo.hour +":"+ _horaInicioIntervalo.min +" e " + _horaFimIntervalo.hour +":"+_horaFimIntervalo.min);
     }

   /*
   if(isValid && (stm.hour < _horaInicioIntervalo.hour ||
                  (stm.hour <= _horaInicioIntervalo.hour && stm.min <= _horaInicioIntervalo.min) ||
                  (stm.hour >= _horaFimIntervalo.hour && stm.min > _horaFimIntervalo.min) ||
                  stm.hour > _horaFimIntervalo.hour
                 ))
     {
         isValid = false;
         _logger.Log("Horário deve estar fora do intervalo de " + _horaInicioIntervaloString + " e " + _horaFimIntervaloString);
        }
   */



   if(isValid)
     {

      if(IsGerenciamentoFinanceiro)
        {

         if(GetTotalLucro() >= MaximoLucroDiario)
           {
            isValid = false;
            _logger.Log("Lucro máximo atingido. R$ " + (string)GetTotalLucro());
           }

         if(GetTotalLucro() <= MaximoPrejuizoDiario)
           {
            isValid = false;
            _logger.Log("Prejuizo máximo atingido. R$ " + (string)GetTotalLucro());
           }

        }

      if(IsParcial && (PrimeiraParcialVolume + SegundaParcialVolume /*+ _terceiraParcialVolume*/) > Volume)
        {
         isValid = false;
         _logger.Log("Valores de parciais inválidos! Verifique-os.");
        }


      if(_isBreakEven)
        {
         if(_breakEven > _breakEvenInicio)
           {
            isValid = false;
            _logger.Log("O Valor do break-even não pode ser maior do que do valor de inicio do mesmo.");
           }
        }


     }


   if(!isValid)
     {
      if(_logger.Last() != _lastTextValidate)
        {
         //SendNotification(_logger.Last());
         //SendMail(_robotName, _logger.Last());
         //MessageBox(_logger.Last());
         Print(_logger.Last());
        }

      _lastTextValidate = _logger.Last();

     }

   return isValid;

  }

// Lucro Total do Dia
double GetTotalLucro()
  {
   return _totalProfitMoney + _totalStopLossMoney;
  }

// Lucro Total do bot na semana
double GetTotalLucro_sem()
  {
   return _totalProfitMoney_sem + _totalStopLossMoney_sem;
  }

// Lucro Total do bot no mes
double GetTotalLucro_mes()
  {
   return _totalProfitMoney_mes + _totalStopLossMoney_mes;
  }

// Lucro Total do bot
double GetTotalLucro_bot()
  {
   return _totalProfitMoney_bot + _totalStopLossMoney_bot;
  }

// Obtem candles
bool GetBuffers()
  {

   ZeroMemory(_rates);
   ArraySetAsSeries(_rates, true);
   ArrayFree(_rates);

   MqlDateTime startDate;
   MqlDateTime stopDate;

   TimeCurrent(startDate);
   stopDate = startDate;

   startDate.hour = GetHoraInicio().hour;
   startDate.min = GetHoraInicio().min;
   startDate.sec = 0;

   int copiedRates = CopyRates(_Symbol, Periodo, 0, 5, _rates);

// Contagem para a proxima barra
   if(IsStats)
     {
      double i;
      int m,s,k;
      m=_rates[0].time + Period() * 60 - TimeCurrent();
      i=m/60.0;
      s=m%60;
      m=(m-m%60)/60;
      proxima_barra = StringFormat("%02i:%02i", m, s);
      //Print(m + " minutes " + s + " seconds left to bar end");
     }


   return copiedRates > 0;

  }

// Set se ja entrou em alguma operação
void SetEntrada(bool value)
  {
   _JaEntrou = value;
  }

// Get se ja entrou em alguma operação
bool GetEntrada()
  {
   return _JaEntrou;
  }

// Periodo estrategia para minutos
int PeriodEnumToMinutes(ENUM_TIMEFRAMES period = PERIOD_CURRENT)
  {
   period = period==PERIOD_CURRENT?(ENUM_TIMEFRAMES)Period():period;
   return PeriodSeconds(period)/60;
  }

// Abre ordem de Compra
void Buy(double price = 0.0, double volume = 0.0, bool isInverted = false)
  {

   if(Validate())
     {

      if(!isInverted && _isInverted)
        {
         Sell(price, volume, true);
         return;
        }

      double stopGain = 0.0;
      double stopLoss = 0.0;
      string msg = "Compra à mercado";

      if(price > 0.0)
        {
         stopGain = NormalizeDouble((price + _stopGain), _Digits);
         stopLoss = NormalizeDouble((price - _stopLoss), _Digits);
         msg = "Compra em " + (string)price +" | SL "+ stopLoss +" | TP "+ stopGain;
        }
      if(IsDebug)
         _logger.Log(msg);

      _trade.Buy(volume > 0 ? volume : Volume, _Symbol, price, stopLoss, stopGain, "("+ NumeroMagico +" - "+ robot_name);

      // Trava por X segundos para nao deixar abrir uma ordem seguida na msma barra
      datetime TC = iTime(Symbol(),_Period,0);
      MqlDateTime TCM;
      TimeToStruct(TC,TCM);
      LiberaProximaOrdem = TC + (PeriodEnumToMinutes(_Period) * (TempoTrava * 60));

      TotalEXEC++;

      //ResetPosition();
     }

  }


// Abre ordem de Compra com Stop
void BuyStop(double price, double volume = 0.0, bool isInverted = false)
  {
   if(Validate())
     {

      if(!isInverted && _isInverted)
        {
         SellStop(price, volume, _isInverted);
         return;
        }

      double stopGain = 0.0;
      double stopLoss = 0.0;
      string msg = "Compra à mercado";

      if(price > 0.0)
        {
         stopGain = NormalizeDouble((price + _stopGain), _Digits);
         stopLoss = NormalizeDouble((price - _stopLoss), _Digits);
         msg = "Compra em " + (string)price +" | SL "+ stopLoss +" | TP "+ stopGain;
        }

      _logger.Log(msg);

      _trade.BuyStop(volume > 0 ? volume : Volume, price, _Symbol, stopLoss, stopGain, ORDER_TIME_DAY, 0, "("+ NumeroMagico +" - "+ robot_name);

     }

  }

// Abre ordem de Venda
void Sell(double price = 0.0, double volume = 0.0, bool isInverted = false)
  {
   if(Validate())
     {

      if(!isInverted && _isInverted)
        {
         Buy(price, volume, _isInverted);
         return;
        }

      double stopGain = 0.0;
      double stopLoss = 0.0;
      string msg = "Venda à mercado";

      if(price > 0.0)
        {
         stopGain = NormalizeDouble((price - _stopGain), _Digits);
         stopLoss = NormalizeDouble((price + _stopLoss), _Digits);
         msg = "Venda em " + (string)price +" | SL "+ stopLoss +" | TP "+ stopGain;
        }

      _logger.Log(msg);

      _trade.Sell(volume > 0 ? volume : Volume, _Symbol, price, stopLoss, stopGain, "BOT " + robot_name +" ("+NumeroMagico+")");

      // Trava por X segundos para nao deixar abrir uma ordem seguida na msma barra
      datetime TC = iTime(Symbol(),_Period,0);
      MqlDateTime TCM;
      TimeToStruct(TC,TCM);
      LiberaProximaOrdem = TC + (PeriodEnumToMinutes(_Period) * (TempoTrava * 60));

      TotalEXEC++;

      //ResetPosition();
     }

  }


// Abre ordem de Venda com Stop
void SellStop(double price, double volume = 0.0, bool isInverted = false)
  {
   if(Validate())
     {

      if(!isInverted && _isInverted)
        {
         BuyStop(price, volume, _isInverted);
         return;
        }

      double stopGain = 0.0;
      double stopLoss = 0.0;
      string msg = "Venda à mercado";

      if(price > 0.0)
        {
         stopGain = NormalizeDouble((price - _stopGain), _Digits);
         stopLoss = NormalizeDouble((price + _stopLoss), _Digits);
         msg = "Venda em " + (string)price +" | SL "+ stopLoss +" | TP "+ stopGain;
        }

      _logger.Log(msg);

      _trade.SellStop(volume > 0 ? volume : Volume, price, _Symbol, stopLoss, stopGain, ORDER_TIME_DAY, 0, "("+ NumeroMagico +" - "+ robot_name);

     }

  }

// Set Stop Gain
void SetStopGain(int value)
  {
   _stopGain = value;
  }

// Get Stop Gain
int GetStopGain()
  {
   return _stopGain;
  }

// Set Stop Loss
void SetStopLoss(int value)
  {
   _stopLoss = value;
  }

// Get Stop Loss
int GetStopLoss()
  {
   return _stopLoss;
  }


// Fecha Ordens Pendentes
void FecharOrdemPendente()
  {
// Ref.: https://www.mql5.com/pt/docs/constants/tradingconstants/orderproperties

   if(TTOrdensPendentes() > 0)
     {

      for(int i=OrdersTotal()-1; i>=0; i--)
        {

         ulong order_ticket = OrderGetTicket(i);

         if(order_ticket > 0)
           {

            if(NumeroMagico == OrderGetInteger(ORDER_MAGIC))
              {
               long ticket = OrderGetInteger(ORDER_TICKET);
               int status = OrderGetInteger(ORDER_STATE);
               _trade.OrderDelete(order_ticket);
              }

           }

        }

     }

  }

// Apaga ordem ao encerrar o dia
void ApagaOrdem()
  {

   ulong order_ticket;
   bool ordem_deletada = false;

   for(int i=OrdersTotal()-1; i>=0; i--)
     {

      if((order_ticket=OrderGetTicket(i)) > 0)
        {

         if(NumeroMagico == OrderGetInteger(ORDER_MAGIC))
           {

            if(ordem_deletada == true)
              {

               bool validation = _trade.OrderDelete(order_ticket);
               if(!validation)
                 {
                  Print("--> Retcode", " =", _trade.ResultRetcode(), "(", _trade.ResultRetcodeDescription(), ")");
                 }

              }

           }

        }

     }

  }

// Total de Ordens Pendente
int TTOrdensPendentes()
  {

   ulong order_ticket;
   int total = 0;

   for(int i=OrdersTotal()-1; i>=0; i--)
     {

      if((order_ticket=OrderGetTicket(i)) > 0)
        {

         if(NumeroMagico == OrderGetInteger(ORDER_MAGIC))
            total++;

        }

     }

   return total;

  }

// Tipo de execução grafica
bool GetRunGraph()
  {

   bool runMode = true;

// Modo real (grafico)
   if(MQLInfoInteger(MQL_TESTER) == false)
      runMode = true;

// Backtest sem grafico
   if(MQLInfoInteger(MQL_TESTER) == true && MQLInfoInteger(MQL_VISUAL_MODE)==false)
      runMode = false;

// Backtest com grafico
   if(MQLInfoInteger(MQL_TESTER) == true && MQLInfoInteger(MQL_VISUAL_MODE)==true)
      runMode = true;

   return runMode;

  }

// Ativar Parciais?
void SetIsParcial(bool flag)
  {
   _isParcial = flag;
  }

// Volume da primeira parcial
void SetPrimeiraParcialVolume(double valor)
  {
   _primeiraParcialVolume = valor;
  }

// Saida da 1ª Parcial
void SetPrimeiraParcialInicio(int valor)
  {
   _primeiraParcialInicio = valor;
  }

// Saida da 2ª Parcial
void SetSegundaParcialInicio(int valor)
  {
   _segundaParcialInicio = valor;
  }

// Volume da segunda parcial
void SetSegundaParcialVolume(double valor)
  {
   _segundaParcialVolume = valor;
  }

// Get parcial 2 Volume
double GetSegundaParcialVolume()
  {
   return _segundaParcialVolume;
  }

// Saida da 3ª Parcial
void SetTerceiraParcialInicio(int valor)
  {
   _terceiraParcialInicio = valor;
  }

// Volume da terceira parcial
void SetTerceiraParcialVolume(double valor)
  {
   _terceiraParcialVolume = valor;
  }

// Desenha Linha das parciais
void DrawParcial(string objName, double price, string text)
  {
   if(ObjectFind(0, objName) > -1 || !_isParcial)
      return;

   ObjectCreate(0, objName, OBJ_HLINE, 0, 0, price);
   ObjectSetInteger(0, objName, OBJPROP_COLOR, _isParcial ? clrOrange : clrDarkGray);
   ObjectSetInteger(0, objName, OBJPROP_BORDER_COLOR, clrBlack);
   ObjectSetInteger(0, objName, OBJPROP_STYLE, STYLE_DASHDOT);
   ObjectSetInteger(0, objName, OBJPROP_WIDTH, 1);
   ObjectSetString(0, objName, OBJPROP_TOOLTIP, text);
   ObjectSetInteger(0, objName, OBJPROP_BACK, true);
   ObjectSetInteger(0, objName, OBJPROP_FILL, true);
   ObjectSetInteger(0, objName, OBJPROP_SELECTED, 1);
   ObjectSetInteger(0, objName, OBJPROP_SELECTABLE, true);
  }

//
void OnDragParcial()
  {
   OnDragLineParcial(_objNamePrimeiraParcial, _primeiraParcialVolume);
   OnDragLineParcial(_objNameSegundaParcial, _segundaParcialVolume);
   OnDragLineParcial(_objNameTerceiraParcial, _terceiraParcialVolume);
  }

// Linha da parcial
void OnDragLineParcial(string objName, double volume)
  {

   for(int i = ObjectsTotal(0, 0, OBJ_HLINE) - 1; i >= 0; i--)
     {
      if(ObjectGetInteger(0, objName, OBJPROP_SELECTED) == 1)
        {
         ObjectSetString(0, objName, OBJPROP_TOOLTIP, "Saída parcial\nPreço " +
                         DoubleToString(ObjectGetDouble(0, objName, OBJPROP_PRICE), _Digits) +
                         "\nVolume " + (string)volume);
        }
     }

  }

// Desenho das linhas das parciais
void ManageDrawParcial()
  {

   if(_isPrimeiraParcialExecuted)
     {
      ClearDraw(_objNamePrimeiraParcial);
     }

   if(_isSegundaParcialExecuted)
     {
      ClearDraw(_objNameSegundaParcial);
     }

   if(_isTerceiraParcialExecuted)
     {
      ClearDraw(_objNameTerceiraParcial);
     }

   if(!HasPositionOpen())
      return;

   if(IsPositionTypeBuy())
     {
      if(!_isPrimeiraParcialExecuted && _primeiraParcialInicio > 0)
        {
         DrawParcial(_objNamePrimeiraParcial, _primeiraParcialInicio,
                     "Saída parcial\nPreço " + (string)(_primeiraParcialInicio) + "\nVolume " + (string)_primeiraParcialVolume);
        }

      if(!_isSegundaParcialExecuted && _segundaParcialInicio > 0)
        {
         DrawParcial(_objNameSegundaParcial, _segundaParcialInicio,
                     "Saída parcial\nPreço " + (string)(_segundaParcialInicio) + "\nVolume " + (string)_segundaParcialVolume);
        }

      if(!_isTerceiraParcialExecuted && _terceiraParcialInicio > 0)
        {
         DrawParcial(_objNameTerceiraParcial, GetPositionPriceOpen() + _terceiraParcialInicio,
                     "Saída parcial\nPreço " + (string)(_terceiraParcialInicio) + "\nVolume " + (string)_terceiraParcialVolume);
        }

      return;

     }

   if(IsPositionTypeSell())
     {

      if(!_isPrimeiraParcialExecuted && _primeiraParcialInicio > 0)
        {
         DrawParcial(_objNamePrimeiraParcial, _primeiraParcialInicio,
                     "Saída parcial\nPreço " + (string)(_primeiraParcialInicio) + "\nVolume " + (string)_primeiraParcialVolume);
        }

      if(!_isSegundaParcialExecuted && _segundaParcialInicio > 0)
        {
         DrawParcial(_objNameSegundaParcial, _segundaParcialInicio,
                     "Saída parcial\nPreço " + (string)(_segundaParcialInicio) + "\nVolume " + (string)_segundaParcialVolume);
        }

      if(!_isTerceiraParcialExecuted && _terceiraParcialInicio > 0)
        {
         DrawParcial(_objNameTerceiraParcial, GetPositionPriceOpen() - _terceiraParcialInicio,
                     "Saída parcial\nPreço " + (string)(_terceiraParcialInicio) + "\nVolume " + (string)_terceiraParcialVolume);
        }

      return;

     }

  }

// Remove as linhas das parciais
void ClearDrawParciais()
  {
   ClearDraw(_objNamePrimeiraParcial);
   ClearDraw(_objNameSegundaParcial);
   ClearDraw(_objNameTerceiraParcial);
  }

// Remove a caixa de statisticas
void ClearDrawStats()
  {
   ClearDraw("Box");
   ClearDraw("Box2");
   ClearDraw("nome_robo");
   ClearDraw("hora_tit");
   ClearDraw("hora_serv");
   ClearDraw("prox_barra");
   ClearDraw("barra_contador");
   ClearDraw("acertos_tit");
   ClearDraw("Lucro_tit");
   ClearDraw("L_acertos_dia");
   ClearDraw("acerto_dia");
   ClearDraw("lucro_dia");
   ClearDraw("L_acertos_sem");
   ClearDraw("acerto_sem");
   ClearDraw("lucro_sem");
   ClearDraw("acertos_mes");
   ClearDraw("acerto_mes");
   ClearDraw("lucro_mes");
   ClearDraw("acertos_total");
   ClearDraw("acerto_tot");
   ClearDraw("lucro_tot");
   ClearDraw("qtd_tit");
   ClearDraw("qtd_cts");
   ClearDraw("rst_tit");
   ClearDraw("rst_pos");
  }

// Descobre se a conta é Hedge
bool IsHedging()
  {
   ENUM_ACCOUNT_MARGIN_MODE m_margin_mode=(ENUM_ACCOUNT_MARGIN_MODE)AccountInfoInteger(ACCOUNT_MARGIN_MODE);
   return m_margin_mode;
  }

// Ativar break Even?
void SetIsBreakEven(bool flag)
  {
   _isBreakEven = flag;
  }

// Define o Break Even
void SetBreakEven(int valor)
  {
   _breakEven = valor;
  }

// Inicio do Break Even
void SetBreakEvenInicio(int valor)
  {
   _breakEvenInicio = valor;
  }


// Seta o magic number
void SetNumberMagic(ulong value)
  {
   _trade.SetExpertMagicNumber(value);
  }

// Lista de horarios de execução da estrategia
string HORARIOS(ENUM_HORARIO time)
  {

   string retorno;
   string hora[109]=
     {
      "09:00","09:05","09:10","09:15","09:20","09:25","09:30","09:35","09:40","09:45","09:50","09:55",
      "10:00","10:05","10:10","10:15","10:20","10:25","10:30","10:35","10:40","10:45","10:50","10:55",
      "11:00","11:05","11:10","11:15","11:20","11:25","11:30","11:35","11:40","11:45","11:50","11:55",
      "12:00","12:05","12:10","12:15","12:20","12:25","12:30","12:35","12:40","12:45","12:50","12:55",
      "13:00","13:05","13:10","13:15","13:20","13:25","13:30","13:35","13:40","13:45","13:50","13:55",
      "14:00","14:05","14:10","14:15","14:20","14:25","14:30","14:35","14:40","14:45","14:50","14:55",
      "15:00","15:05","15:10","15:15","15:20","15:25","15:30","15:35","15:40","15:45","15:50","15:55",
      "16:00","16:05","16:10","16:15","16:20","16:25","16:30","16:35","16:40","16:45","16:50","16:55",
      "17:00","17:05","17:10","17:15","17:20","17:25","17:30","17:35","17:40","17:45","17:50","17:55",
      "18:00"
     };

   for(int i=0; i<=108; i++)
     {
      if(i==time)
        {
         retorno=hora[i];
         break;
        }
     }
   return(retorno);

  }

// Retorna o Numero da semana
int WeekNumber(MqlDateTime &dt)
  {
   int iDay  = (dt.day_of_week + 6) % 7 + 1;         // convert day to standard index (1=Mon,...,7=Sun)
   int iWeek = (dt.day_of_year - iDay + 10) / 7;     // calculate standard week number
   return iWeek;
  }

// Criar os objetos do painel - Texto
void ObjCreate_texto(string nome_obj, int POSX, int POSY, string texto, string fonte, int fontsize, color cor)
  {
   ObjectCreate(0, nome_obj,OBJ_LABEL,0,0,0);
   ObjectSetInteger(0, nome_obj,OBJPROP_XDISTANCE, POSX);
   ObjectSetInteger(0, nome_obj,OBJPROP_YDISTANCE, POSY);
   ObjectSetInteger(0, nome_obj,OBJPROP_CORNER, CORNER_LEFT_LOWER);
   ObjectSetString(0, nome_obj,OBJPROP_TEXT, texto);
   ObjectSetString(0, nome_obj,OBJPROP_FONT, fonte);
   ObjectSetInteger(0, nome_obj,OBJPROP_FONTSIZE, fontsize);
   ObjectSetInteger(0, nome_obj,OBJPROP_COLOR, cor);
   ObjectSetInteger(0, nome_obj,OBJPROP_SELECTABLE, false);
   ObjectSetInteger(0, nome_obj,OBJPROP_SELECTED, false);
   ObjectSetInteger(0, nome_obj,OBJPROP_HIDDEN, false);
   ObjectSetInteger(0, nome_obj,OBJPROP_BACK, false);
  }


// Janela de estatisticas
bool Box_Creation(int yPainelMax = 200)
  {

   ResetLastError();

// Quadrado grande
   ObjectCreate(0,"Box",OBJ_RECTANGLE_LABEL,0,0,0);
   ObjectSetInteger(0,"Box",OBJPROP_XDISTANCE,10);
   ObjectSetInteger(0,"Box",OBJPROP_YDISTANCE,yPainelMax);
   ObjectSetInteger(0,"Box",OBJPROP_XSIZE,250);
   ObjectSetInteger(0,"Box",OBJPROP_YSIZE,yPainelMax-10);
   ObjectSetInteger(0,"Box",OBJPROP_BGCOLOR,clrBlack);
   ObjectSetInteger(0,"Box",OBJPROP_BORDER_TYPE,BORDER_FLAT);
   ObjectSetInteger(0,"Box",OBJPROP_STYLE,STYLE_SOLID);
   ObjectSetInteger(0,"Box",OBJPROP_WIDTH,2);
   ObjectSetInteger(0,"Box",OBJPROP_BORDER_COLOR,clrLightSlateGray);
   ObjectSetInteger(0,"Box",OBJPROP_CORNER,CORNER_LEFT_LOWER);
   ObjectSetInteger(0,"Box",OBJPROP_COLOR,clrLightSlateGray);
   ObjectSetInteger(0,"Box",OBJPROP_SELECTABLE,false);
   ObjectSetInteger(0,"Box",OBJPROP_SELECTED,false);
   ObjectSetInteger(0,"Box",OBJPROP_HIDDEN,false);
   ObjectSetInteger(0,"Box",OBJPROP_BACK,false);


// Faixa topo titulo
   ObjectCreate(0,"Box2",OBJ_RECTANGLE_LABEL,0,0,0);
   ObjectSetInteger(0,"Box2",OBJPROP_XDISTANCE,10);
   ObjectSetInteger(0,"Box2",OBJPROP_YDISTANCE,yPainelMax);
   ObjectSetInteger(0,"Box2",OBJPROP_XSIZE,250);
   ObjectSetInteger(0,"Box2",OBJPROP_YSIZE,17);
   ObjectSetInteger(0,"Box2",OBJPROP_BGCOLOR,clrLightSlateGray);
   ObjectSetInteger(0,"Box2",OBJPROP_STYLE,STYLE_SOLID);
   ObjectSetInteger(0,"Box2",OBJPROP_CORNER,CORNER_LEFT_LOWER);
   ObjectSetInteger(0,"Box2",OBJPROP_COLOR,clrGray);
   ObjectSetInteger(0,"Box2",OBJPROP_SELECTABLE,false);
   ObjectSetInteger(0,"Box2",OBJPROP_SELECTED,false);
   ObjectSetInteger(0,"Box2",OBJPROP_HIDDEN,false);
   ObjectSetInteger(0,"Box2",OBJPROP_BACK,false);


// Texto Nome do robo
   ObjCreate_texto("nome_robo", 20, yPainelMax-2, robot_name +"   -   Magic N.:"+ NumeroMagico, "Verdana", 8, clrWhite);
   ObjCreate_texto("hora_tit", 15, yPainelMax-23, "Hora Serv.", "Verdana", 8, clrWhite); // Label hora do servidor
   ObjCreate_texto("hora_serv", 15, yPainelMax-35, TimeToString(TimeCurrent(),TIME_SECONDS), "Verdana", 12, clrOrangeRed); // Horario do servidor
   ObjCreate_texto("prox_barra", 190, yPainelMax-23, "Prox. Barra", "Verdana", 8, clrWhite); // Label proxima vela
   ObjCreate_texto("barra_contador", 200, yPainelMax-35, proxima_barra, "Verdana", 12, clrWhite); // Proxima vela


   ObjCreate_texto("acertos_tit", 88, yPainelMax-60, "Total de Acertos", "Verdana", 7, clrWhite); // Label Total de Acertos
   ObjCreate_texto("Lucro_tit", 200, yPainelMax-60, "Lucro", "Verdana", 7, clrWhite); // Label Lucro



   string AcertoDia = _totalOrdensVolume < 1 ? "0 / 0 (0%)" : _totalOrdensAcertos +" / "+ _totalOrdensVolume +" ("+ (_totalOrdensAcertos * 100) / _totalOrdensVolume +"%)";

   ObjCreate_texto("L_acertos_dia", 15, yPainelMax-75, "Dia", "Verdana", 7, clrWhite); // Label Total de Acertos Dia
   ObjCreate_texto("acerto_dia", 80, yPainelMax-75, AcertoDia, "Verdana", 8, clrDodgerBlue); // Acertos no Dia
   ObjCreate_texto("lucro_dia", 200, yPainelMax-75, GetTotalLucro(), "Verdana", 8, clrDodgerBlue); // Lucro no Dia



   string AcertoSem = _totalOrdensVolume_sem < 1 ? "0 / 0 (0%)" : _totalOrdensAcertos_sem +" / "+ _totalOrdensVolume_sem +" ("+ (_totalOrdensAcertos_sem * 100) / _totalOrdensVolume_sem +"%)";

   ObjCreate_texto("L_acertos_sem", 15, yPainelMax-90, "Semana", "Verdana", 7, clrWhite); // Label Total de Acertos Semana
   ObjCreate_texto("acerto_sem", 80, yPainelMax-90, AcertoSem, "Verdana", 8, clrDodgerBlue); // Acertos na Semana
   ObjCreate_texto("lucro_sem", 200, yPainelMax-90, GetTotalLucro_sem(), "Verdana", 8, clrDodgerBlue); // Lucro na Semana



   string AcertoMes = _totalOrdensVolume_mes < 1 ? "0 / 0 (0%)" : _totalOrdensAcertos_mes +" / "+ _totalOrdensVolume_mes +" ("+ (_totalOrdensAcertos_mes * 100) / _totalOrdensVolume_mes +"%)";

   ObjCreate_texto("acertos_mes", 15, yPainelMax-105, "Mês", "Verdana", 7, clrWhite); // Label Total de Acertos Mês
   ObjCreate_texto("acerto_mes", 80, yPainelMax-105, AcertoMes, "Verdana", 8, clrDodgerBlue); // Acertos no Mês
   ObjCreate_texto("lucro_mes", 200, yPainelMax-105, GetTotalLucro_mes(), "Verdana", 8, clrDodgerBlue); // Lucro no Mês



   string AcertoTotal = _totalOrdensVolume_bot < 1 ? "0 / 0 (0%)" : _totalOrdensAcertos_bot +" / "+ _totalOrdensVolume_bot +" ("+ (_totalOrdensAcertos_bot * 100) / _totalOrdensVolume_bot +"%)";

   ObjCreate_texto("acertos_total", 15, yPainelMax-120, "Total", "Verdana", 7, clrWhite); // Label Total de Acertos
   ObjCreate_texto("acerto_tot", 80, yPainelMax-120, AcertoTotal, "Verdana", 8, clrDodgerBlue); // Acertos no total
   ObjCreate_texto("lucro_tot", 200, yPainelMax-120, GetTotalLucro_bot(), "Verdana", 8, clrDodgerBlue); // Lucro no Total


   ObjCreate_texto("qtd_tit", 30, yPainelMax-143, "Qtd Ct(s)", "Verdana", 8, clrWhite); // Label Qtd de contratos em aberto
   ObjCreate_texto("qtd_cts", 45, yPainelMax-156, GetPositionVolume(), "Verdana", 12, clrWhite); // Qtd de contratos em aberto

   ObjCreate_texto("rst_tit", 165, yPainelMax-143, "Resultado", "Verdana", 8, clrWhite); // Label Qtd de contratos em aberto
   ObjCreate_texto("rst_pos", 145, yPainelMax-156, "R$ "+ GetPositionProfit(), "Verdana", 12, clrWhite); // Qtd de contratos em aberto



   return true;

  }
//+------------------------------------------------------------------+







//+------------------------------------------------------------------+
//| ESTRATEGIA                                                       |
//+------------------------------------------------------------------+
void OnTickHandler()
  {
   double ticksize = SymbolInfoDouble(_Symbol,SYMBOL_TRADE_TICK_SIZE);
   double _lastprice = GetLastPrice();

   double _MEDIA = NormalizeDouble(_MA1[0], 0);



   double _DISTANCIA;
   if(_MEDIA > _lastprice)
      _DISTANCIA = _MEDIA - _lastprice;
   else
      _DISTANCIA = _lastprice - _MEDIA;

//_logger.Log("DISTANCIA "+ _DISTANCIA);



   if(!HasPositionOpen() && TimeCurrent() >= LiberaProximaOrdem && (TotalEXEC < QTDEXEC || QTDEXEC == 0))
     {


      // Compra
      if(_MEDIA > _lastprice && _DISTANCIA >= DistMAMin && _DISTANCIA <= DistMAMax)
        {

         // Saida parcial (EXEMPLO)
         if(IsParcial)
           {
            SetStopLoss(StopLossEmTicks * 2);
            SetPrimeiraParcialInicio(_lastprice + StopGainEmTicks);


            if(SegundaParcialVolume > 0)
              {
               double SAIDA2PARC = (_lastprice + StopGainEmTicks + SegundaParcialInicio);
               SetSegundaParcialInicio(SAIDA2PARC);
              }

            if(TerceiraParcialVolume > 0)
              {
               double SAIDA3PARC = (_lastprice + StopGainEmTicks + SegundaParcialInicio + TerceiraParcialInicio);
               SetTerceiraParcialInicio(SAIDA3PARC);
              }

            SetStopGain((StopGainEmTicks * 2) + SegundaParcialInicio + TerceiraParcialInicio + ticksize);

           }
         else
           {
            SetStopLoss(StopLossEmTicks);
            SetStopGain(StopGainEmTicks);
           }

         // Abre a ordem de compra
         Buy(_lastprice);

        }






      // Venda
      if(_MEDIA < _lastprice && _DISTANCIA >= DistMAMin && _DISTANCIA <= DistMAMax)
        {

         // Saida parcial
         if(IsParcial)
           {

            SetStopLoss(StopLossEmTicks * 2);
            SetPrimeiraParcialInicio(_lastprice - StopGainEmTicks);


            if(SegundaParcialVolume > 0)
              {
               double SAIDA2PARC = (_lastprice - StopGainEmTicks - SegundaParcialInicio);
               SetSegundaParcialInicio(SAIDA2PARC);
              }

            if(TerceiraParcialVolume > 0)
              {
               double SAIDA3PARC = (_lastprice - StopGainEmTicks - SegundaParcialInicio - TerceiraParcialInicio);
               SetTerceiraParcialInicio(SAIDA3PARC);
              }

            SetStopGain((StopGainEmTicks * 2) + SegundaParcialInicio + TerceiraParcialInicio + ticksize);

           }
         else
           {
            SetStopLoss(StopLossEmTicks);
            SetStopGain(StopGainEmTicks);
           }

         // Abre ordem de Venda
         Sell(_lastprice);

        }


     }

  }
//+------------------------------------------------------------------+
